<?php
namespace Drupal\zohomail\Helper;


use Drupal\zohomail\Helper\ZConstants;

class ZohoMailApi {
	private $domain;
	private $accountId;
	private $config;
	private $timestamp;

	private function getZohoMailUrl() {
		return "https://mail.".ZConstants::$domains[$this->domain];
	}
	private function getAccountsUrl() {
		return "https://accounts.".ZConstants::$domains[$this->domain];
	}
	public function __construct($config) {
		$this->config = $config;
		$this->domain = $config->get('domain');
		$this->timestamp = $config->get('timestamp');
		$this->accountId = $config->get('account_id');
	}
	public function sendMail($mail_data)
	{
		$accountId = $this->accountId;
		return $this->sendZohoMail($mail_data,$accountId);
	}
	public function sendTestMail($mail_data,$accountId)
	{
		return $this->sendZohoMail($mail_data,$accountId);
	}
	public function sendZohoMail($mail_data,$accountId) {
		$responseObj = json_decode('{}');
		$urlToSend = $this->getZohoMailUrl().'/api/accounts/'.$accountId.'/messages';	
		$curl = curl_init();
		curl_setopt_array($curl, array(
				CURLOPT_URL => $urlToSend,
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_ENCODING => "",
				CURLOPT_MAXREDIRS => 10,
				CURLOPT_TIMEOUT => 30,
				CURLOPT_SSLVERSION => CURL_SSLVERSION_TLSv1_2,
				CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
				CURLOPT_CUSTOMREQUEST => "POST",
				CURLOPT_POSTFIELDS => json_encode($mail_data),
				CURLOPT_HTTPHEADER => array(
				"accept: application/json",
				"authorization: Bearer ".$this->getAccessToken(),
				"cache-control: no-cache",
				"content-type: application/json",
				"User-Agent: DrupalPlugin"
			),
		));
		$response = curl_exec($curl);
		$err = curl_error($curl);
		curl_close($curl);
		$httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		if($httpcode == '200' || $httpcode == '201') {
			$responseObj->result = "success";
			$responseObj->details = $response;
		}else{
			$responseObj->result ="error";
			$responseObj->details = $response;
		}
		return $responseObj;
	}
	public function getZohoMailAccounts() {
		$responseObj = json_decode('{}',true);
		$urlToSend = $this->getZohoMailUrl()."/api/accounts";	
		$curl = curl_init();
		curl_setopt_array($curl, array(
				CURLOPT_URL => $urlToSend,
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_ENCODING => "",
				CURLOPT_MAXREDIRS => 10,
				CURLOPT_TIMEOUT => 30,
				CURLOPT_SSLVERSION => CURL_SSLVERSION_TLSv1_2,
				CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
				CURLOPT_CUSTOMREQUEST => "GET",
				CURLOPT_HTTPHEADER => array(
				"accept: application/json",
				"authorization: Bearer ".$this->getAccessToken(),
				"cache-control: no-cache",
				"content-type: application/json",
				"User-Agent: MagentoPlugin"
			),
		));
		$response = curl_exec($curl);
		$err = curl_error($curl);
		curl_close($curl);
		$httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		if($httpcode == '200' || $httpcode == '201') {
			$responseObj["result"] ="success";
			$responseObj["detail"]=json_decode($response);
		}else{
			$responseObj["result"] ="error";
			$responseObj["detail"]=json_decode($response);
		}
		
		return $responseObj;
	}

	private function getAccessToken() {
		if( !empty($this->config->get('timestamp')) && time() - $this->config->get('timestamp') > 3000) {
			$url = $this->getAccountsUrl()."/oauth/v2/token?refresh_token=".base64_decode($this->config->get('refresh_token'))."&client_id=".base64_decode($this->config->get('client_id'))."&client_secret=".base64_decode($this->config->get('client_secret'))."&redirect_uri=".$this->config->get('callback_url')."&grant_type=refresh_token";
			$curl = curl_init();
            curl_setopt_array($curl, array(
				CURLOPT_URL => $url,
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_ENCODING => "",
				CURLOPT_MAXREDIRS => 10,
				CURLOPT_TIMEOUT => 30,
				CURLOPT_SSLVERSION => CURL_SSLVERSION_TLSv1_2,
				CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
				CURLOPT_CUSTOMREQUEST => "POST"
		    ));
            $response = curl_exec($curl);
            $err = curl_error($curl);
            curl_close($curl);
           
            $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
			
			if($httpcode == '200'){
				$body = json_decode($response);
				$config = \Drupal::configFactory()->getEditable('zohomail.settings');
				$config->set('access_token', base64_encode($body->access_token))
						->set('timestamp',time())
						->save();
				return $body->access_token;
			}
			else {
				return null;
			}
			
		}
		else{
			return base64_decode($this->config->get('access_token'));
		}
	}
	public function getZohoMailAccountDetails() {
        $response = $this->getZohoMailAccounts();
        $emailDetail = array();
        $emailArr = array();
        $accountId = '';
        if($response["result"] === "success") {
            $jsonbodyAccounts = $response["detail"];
            
            for($i=0;$i<count($jsonbodyAccounts->data);$i++)
			{
                $emailData = $jsonbodyAccounts->data[$i];
                if(!empty($emailData->sendMailDetails))
                {
                    $accountId = $jsonbodyAccounts->data[$i]->accountId;
                    for($j=0;$j<count($jsonbodyAccounts->data[$i]->sendMailDetails);$j++) {
                        array_push($emailArr,$jsonbodyAccounts->data[$i]->sendMailDetails[$j]->fromAddress);
                    }
                }
			}
          
        }
        $emailDetail['account_id'] = $accountId;
        $emailDetail["sendmail_details"] = $emailArr;

        return $emailDetail;
    }

	public function uploadAttachment($attachments) {
		$accountId = $this->config->get('account_id');
		$boundary = uniqid();
		$eol = "\r\n"; 
		$body = '';
		foreach ($attachments as $attachment) {
			$isFile = false;
			$filename = '';
			$filecontent = '';
			$filemime = '';
			if (!empty($attachment['filepath']) && file_exists($attachment['filepath'])) {
			  $fileContent = file_get_contents($attachment['filepath']);
			  $filename = basename($attachment['filepath']);
			  $filemime = mime_content_type($fileContent);
			  
			  $isFile =true;
			}
			elseif (!empty($attachment['filecontent']) && !empty($attachment['filename'])) {
				$fileContent =  $attachment['filecontent'];
				$filename = $attachment['filename'];
				$filemime =$attachment['filemime'];
				$isFile =true;
			}
			if($isFile) {
				$body .= '--' . $boundary . $eol;
				$body .= 'Content-Disposition: form-data; name="attach"; filename="' . $filename . '"' . $eol;
				$body .= 'Content-Type: ' . $filemime . $eol . $eol;
				$body .= $fileContent . $eol;
				
			}
		}
		$body .= '--' . $boundary . '--' . $eol;
		$headers = [
			'Content-Type: multipart/form-data; boundary=' . $boundary,
			'Content-Length: ' . strlen($body),
			"authorization: Bearer ".$this->getAccessToken(),
			"User-Agent: MagentoPlugin"
		];
		$context = stream_context_create([
			'http' => [
				'method' => 'POST',
				'header' => implode("\r\n", $headers),
				'content' => $body
			]
		]);
		
		$response = file_get_contents($this->getZohoMailUrl().'/api/accounts/'.$accountId.'/messages/attachments?uploadType=multipart', false, $context);
		return $response;
	}

	
   
}
	

	